@@ -22,6 +22,7 @@
 
 #include <config.h>
 #include <string.h>
+#include <signal.h>
 #include <stdlib.h>
 #include <dbus/dbus.h>
 #include "gdaemonvfs.h"
@@ -80,8 +81,11 @@ g_daemon_vfs_finalize (GObject *object)
 
   vfs = G_DAEMON_VFS (object);
 
-  g_hash_table_destroy (vfs->from_uri_hash);
-  g_hash_table_destroy (vfs->to_uri_hash);
+  if (vfs->from_uri_hash)
+    g_hash_table_destroy (vfs->from_uri_hash);
+  
+  if (vfs->to_uri_hash)
+    g_hash_table_destroy (vfs->to_uri_hash);
 
   g_strfreev (vfs->supported_uri_schemes);
 
@@ -91,6 +95,9 @@ g_daemon_vfs_finalize (GObject *object)
       dbus_connection_unref (vfs->async_bus);
     }
 
+  if (vfs->wrapped_vfs)
+    g_object_unref (vfs->wrapped_vfs);
+  
   /* must chain up */
   G_OBJECT_CLASS (g_daemon_vfs_parent_class)->finalize (object);
 }
@@ -105,20 +112,34 @@ get_mountspec_from_uri (GDaemonVfs *vfs,
   char *path;
   GVfsUriMapper *mapper;
   char *scheme;
-
+  GVfsUriMountInfo *info;
+  
   scheme = g_uri_get_scheme (uri);
   if (scheme == NULL)
     return FALSE;
   
   spec = NULL;
+  path = NULL;
   
   mapper = g_hash_table_lookup (vfs->from_uri_hash, scheme);
-  if (mapper == NULL ||
-      !g_vfs_uri_mapper_from_uri (mapper, uri, &spec, &path))
+  
+  if (mapper)
+    {
+      info = g_vfs_uri_mapper_from_uri (mapper, uri);
+      if (info != NULL)
+	{
+	  spec = g_mount_spec_new_from_data (info->keys, NULL);
+	  path = info->path;
+	  /* We took over ownership, custom free: */
+	  g_free (info);
+	}
+    }
+  
+  if (spec == NULL)
     {
       GDecodedUri *decoded;
       
-      decoded = g_decode_uri (uri);
+      decoded = g_vfs_decode_uri (uri);
       if (decoded)
 	{
 	  spec = g_mount_spec_new (decoded->scheme);
@@ -138,7 +159,7 @@ get_mountspec_from_uri (GDaemonVfs *vfs,
 	  
 	  path = g_strdup (decoded->path);
 
-	  g_decoded_uri_free (decoded);
+	  g_vfs_decoded_uri_free (decoded);
 	}
     }
   
@@ -162,18 +183,32 @@ g_daemon_vfs_init (GDaemonVfs *vfs)
   GVfsUriMapper *mapper;
   int i;
   
-  g_assert (the_vfs == NULL);
-  the_vfs = vfs;
-
-  vfs->wrapped_vfs = g_vfs_get_local ();
-
-  if (g_thread_supported ())
-    dbus_threads_init_default ();
-  
   vfs->async_bus = dbus_bus_get_private (DBUS_BUS_SESSION, NULL);
 
-  if (vfs->async_bus)
-    _g_dbus_connection_integrate_with_main (vfs->async_bus);
+  if (vfs->async_bus == NULL)
+    return; /* Not supported, return here and return false in vfs_is_active() */
+
+  g_assert (the_vfs == NULL);
+  the_vfs = vfs;
+  
+  if (g_thread_supported ())
+    dbus_threads_init_default ();
+
+  /* We disable SIGPIPE globally. This is sort of bad
+     for s library to do since its a global resource.
+     However, without this there is no way to be able
+     to handle mount daemons dying without client apps
+     crashing, which is much worse.
+
+     I blame Unix, there really should be a portable
+     way to do this on all unixes, but there isn't,
+     even for somewhat modern ones like solaris.
+  */
+  signal (SIGPIPE, SIG_IGN);
+  
+  vfs->wrapped_vfs = g_vfs_get_local ();
+
+  _g_dbus_connection_integrate_with_main (vfs->async_bus);
 
   g_io_modules_ensure_loaded (GVFS_MODULE_DIR);
 
@@ -272,7 +307,12 @@ _g_daemon_vfs_get_uri_for_mountspec (GMountSpec *spec,
   uri = NULL;
   mapper = g_hash_table_lookup (the_vfs->to_uri_hash, type);
   if (mapper)
-    uri = g_vfs_uri_mapper_to_uri (mapper, spec, path, allow_utf8);
+    {
+      GVfsUriMountInfo info;
+      info.keys = spec->items;
+      info.path = path;
+      uri = g_vfs_uri_mapper_to_uri (mapper, &info, allow_utf8);
+    }
 
   if (uri == NULL)
     {
@@ -294,7 +334,7 @@ _g_daemon_vfs_get_uri_for_mountspec (GMountSpec *spec,
       else
 	decoded.path = path;
       
-      uri = g_encode_uri (&decoded, FALSE);
+      uri = g_vfs_encode_uri (&decoded, FALSE);
     }
   
   return uri;
@@ -311,15 +351,21 @@ _g_daemon_vfs_mountspec_get_uri_scheme (GMountSpec *spec)
 
   scheme = NULL;
   if (mapper)
-    scheme = g_vfs_uri_mapper_to_uri_scheme (mapper, spec);
+    {
+      GVfsUriMountInfo info;
+      
+      info.keys = spec->items;
+      info.path = "/";
+      
+      scheme = g_vfs_uri_mapper_to_uri_scheme (mapper, &info);
+    }
   
   if (scheme == NULL)
     scheme = type;
-
+  
   return scheme;
 }
 
-
 static void
 fill_supported_uri_schemes (GDaemonVfs *vfs)
 {
@@ -381,7 +427,14 @@ fill_supported_uri_schemes (GDaemonVfs *vfs)
       mapper = g_hash_table_lookup (vfs->to_uri_hash, type);
     
       if (mapper)
-        scheme = g_vfs_uri_mapper_to_uri_scheme (mapper, spec);
+	{
+	  GVfsUriMountInfo info;
+	  
+	  info.keys = spec->items;
+	  info.path = "/";
+	  
+	  scheme = g_vfs_uri_mapper_to_uri_scheme (mapper, &info);
+	}
 
       if (scheme == NULL)
         scheme = type;
@@ -465,6 +518,30 @@ lookup_mount_info_in_cache (GMountSpec *spec,
   return info;
 }
 
+void
+_g_daemon_vfs_invalidate_dbus_id (const char *dbus_id)
+{
+  GMountInfo *info;
+  GList *l, *next;
+
+  G_LOCK (mount_cache);
+  info = NULL;
+  for (l = the_vfs->mount_cache; l != NULL; l = next)
+    {
+      GMountInfo *mount_info = l->data;
+      next = l->next;
+
+      if (strcmp (mount_info->dbus_id, dbus_id) == 0)
+	{
+	  the_vfs->mount_cache = g_list_delete_link (the_vfs->mount_cache, l);
+	  g_mount_info_unref (mount_info);
+	}
+    }
+  
+  G_UNLOCK (mount_cache);
+}
+
+
 static GMountInfo *
 handler_lookup_mount_reply (DBusMessage *reply,
 			    GError **error)
@@ -668,16 +745,11 @@ _g_daemon_vfs_get_async_bus (void)
   return the_vfs->async_bus;
 }
 
-static const char *
-g_daemon_vfs_get_name (GVfs *vfs)
+static gboolean
+g_daemon_vfs_is_active (GVfs *vfs)
 {
-  return "gvfs";
-}
-
-static int
-g_daemon_vfs_get_priority (GVfs *vfs)
-{
-  return 10;
+  GDaemonVfs *daemon_vfs = G_DAEMON_VFS (vfs);
+  return daemon_vfs->async_bus != NULL;
 }
 
 static void
@@ -698,8 +770,11 @@ g_daemon_vfs_class_init (GDaemonVfsClass *class)
   object_class->finalize = g_daemon_vfs_finalize;
 
   vfs_class = G_VFS_CLASS (class);
-  vfs_class->get_name = g_daemon_vfs_get_name;
-  vfs_class->get_priority = g_daemon_vfs_get_priority;
+
+  vfs_class->name = "gvfs";
+  vfs_class->priority = 10;
+  
+  vfs_class->is_active = g_daemon_vfs_is_active;
   vfs_class->get_file_for_path = g_daemon_vfs_get_file_for_path;
   vfs_class->get_file_for_uri = g_daemon_vfs_get_file_for_uri;
   vfs_class->get_supported_uri_schemes = g_daemon_vfs_get_supported_uri_schemes;
@@ -708,11 +783,16 @@ g_daemon_vfs_class_init (GDaemonVfsClass *class)
 
 /* Module API */
 
+void g_vfs_uri_mapper_smb_register (GIOModule *module);
+
 void
 g_io_module_load (GIOModule *module)
 {
   g_daemon_vfs_register_type (G_TYPE_MODULE (module));
   g_daemon_volume_monitor_register_types (G_TYPE_MODULE (module));
+
+  g_vfs_uri_mapper_register (module);
+  g_vfs_uri_mapper_smb_register (module);
 }
 
 void
