@@ -22,6 +22,8 @@
 
 #include <config.h>
 
+#include <string.h>
+#include <strings.h>
 #include <unistd.h>
 #include <errno.h>
 #include <stdlib.h>
@@ -87,7 +89,7 @@ dbus_connection_send_fd (DBusConnection *connection,
     {
       g_set_error (error, G_IO_ERROR,
 		   G_IO_ERROR_FAILED,
-		   _("No fd passing socket availible"));
+		   _("No fd passing socket available"));
       return FALSE;
     }
 
